@@ -1,15 +1,22 @@
 import os
 from configparser import ConfigParser
-from argparse import Namespace, ArgumentError
+from argparse import ArgumentParser, Namespace
 from datetime import date, timedelta
 from io import StringIO
 
 from worklog.parser import get_arg_parser
-from worklog.utils import configure_logger, LOG_LEVELS, CONFIG_FILES
+from worklog.utils import (
+    configure_logger,
+    LOG_LEVELS,
+    CONFIG_FILES,
+    calc_log_time,
+)
 from worklog.log import Log
 
 
-def dispatch(log: Log, cli_args: Namespace, cfg: ConfigParser) -> None:
+def dispatch(
+    log: Log, parser: ArgumentParser, cli_args: Namespace, cfg: ConfigParser
+) -> None:
     """
     Dispatch request to Log instance based on CLI arguments and
     configuration values.
@@ -24,11 +31,18 @@ def dispatch(log: Log, cli_args: Namespace, cfg: ConfigParser) -> None:
                 force=cli_args.force,
             )
     elif cli_args.subcmd == "task":
+        if cli_args.auto_close == True and cli_args.type != "start":
+            raise parser.error(
+                '--auto-close is only allowed if the type is set to "start"',
+            )
         if cli_args.type in ["start", "stop"]:
             if cli_args.id is None:
-                raise ArgumentError(
-                    cli_args.id, "--id is required when a new task is started/stopped"
+                raise parser.error(
+                    "--id is required when a new task is started/stopped"
                 )
+            if cli_args.type == "start" and cli_args.auto_close:
+                commit_dt = calc_log_time(cli_args.offset_minutes, cli_args.time)
+                log.stop_active_tasks(commit_dt)
             log.commit(
                 "task",
                 cli_args.type,
@@ -40,9 +54,7 @@ def dispatch(log: Log, cli_args: Namespace, cfg: ConfigParser) -> None:
             log.list_tasks()
         elif cli_args.type == "report":
             if cli_args.id is None:
-                raise ArgumentError(
-                    cli_args.id, "--id is required when requesting a report"
-                )
+                raise parser.error("--id is required when requesting a report")
             log.task_report(cli_args.id)
     elif cli_args.subcmd == "status":
         hours_target = float(cfg.get("workday", "hours_target"))
@@ -51,6 +63,8 @@ def dispatch(log: Log, cli_args: Namespace, cfg: ConfigParser) -> None:
         query_date = date.today()
         if cli_args.yesterday:
             query_date -= timedelta(days=1)
+        elif cli_args.date:
+            query_date = cli_args.date.date()
         log.status(hours_target, hours_max, query_date=query_date, fmt=fmt)
     elif cli_args.subcmd == "doctor":
         log.doctor()
@@ -63,6 +77,8 @@ def dispatch(log: Log, cli_args: Namespace, cfg: ConfigParser) -> None:
             log.log(cli_args.number, use_pager, categories)
         else:
             log.log(-1, use_pager, categories)
+    elif cli_args.subcmd == "report":
+        log.report(cli_args.month_from, cli_args.month_to)
 
 
 def run() -> None:
@@ -92,7 +108,7 @@ def run() -> None:
     worklog_fp = os.path.expanduser(cfg.get("worklog", "path"))
     log = Log(worklog_fp)
 
-    dispatch(log, cli_args, cfg)
+    dispatch(log, parser, cli_args, cfg)
 
 
 if __name__ == "__main__":
